/*!
* Copyright Adaptavist 2022 (c) All rights reserved
*/
import { Request, Response, BaseApiCore, constructUrl, executeHttpCall } from '@managed-api/commons-core';
import { constructHeaders, errorMessageBuilder } from './common';
import { customErrorHandlers, errorStrategyBuilderConstructor } from './errorStrategy';
export class BitbucketOnPremiseApiCore extends BaseApiCore {
    constructor(options) {
        super();
        this.options = options;
        if (!this.options?.errorStrategy && this.options?.errorStrategy !== null) {
            this.setGlobalErrorStrategy((builder) => builder.retryOnRateLimiting(-1));
        }
        this.Repository = new RepositoryGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Project = new ProjectGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.User = new UserGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.All = new AllGroup(this.Repository, this.Project, this.User);
    }
    setGlobalErrorStrategy(errorStrategy) {
        this.options = {
            ...this.options ?? {},
            errorStrategy
        };
    }
    buildResponse(url, status, statusText, headers, body) {
        const response = new Response(url, status, statusText, headers, body);
        response.setBufferImplementation(this.implementation);
        return response;
    }
    getGlobalErrorStrategy() {
        return this.options?.errorStrategy;
    }
}
class AllGroup {
    constructor(Repository, Project, User) {
        this.Repository = Repository;
        this.Project = Project;
        this.User = User;
    }
    /**
     * This is a paged API. This API can also be invoked via a user-centric URL when addressing repositories in personal projects.
     * Retrieve a page of commits from a given starting commit or "between" two commits.
     * If no explicit commit is specified, the tip of the repository's default branch is assumed.
     * Commits may be identified by branch or tag name or by ID.
     * A path may be supplied to restrict the returned commits to only those which affect that path.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Alternative usage: Repository.Commit.getRepositoryCommits
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp222
     * @param options Request options.
     */
    async getRepositoryCommits(options) {
        return this.Repository.Commit.getRepositoryCommits(options);
    }
    /**
     * This API can also be invoked via a user-centric URL when addressing repositories in personal projects.
     * Retrieve a single commit identified by its ID. In general, that ID is a SHA1. From 2.11, ref names like "refs/heads/master" are no longer accepted by this resource.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Alternative usage: Repository.Commit.getCommit
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp224
     * @param options Request options.
     */
    async getRepositoryCommit(options) {
        return this.Repository.Commit.getCommit(options);
    }
    /**
     * Retrieve a page of users.
     * The authenticated user must have the LICENSED_USER permission to call this resource.
     * Alternative usage: User.getUsers
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp17
     * @param options Request options.
     */
    async getUsers(options) {
        return this.User.getUsers(options);
    }
    /**
     * Deletes the specified user, removing them from the system. This also removes any permissions that may have been granted to the user.
     * A user may not delete themselves, and a user with ADMIN permissions may not delete a user with SYS_ADMINpermissions.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Alternative usage: User.deleteUser
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp18
     * @param options Request options.
     */
    async deleteUser(options) {
        return this.User.deleteUser(options);
    }
    /**
     * Creates a new user from the assembled query parameters.
     * The default group can be used to control initial permissions for new users, such as granting users the ability to login or providing read access to certain projects or repositories. If the user is not added to the default group, they may not be able to login after their account is created until explicit permissions are configured.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Alternative usage: User.createUser
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp19
     * @param options Request options.
     */
    async createUser(options) {
        return this.User.createUser(options);
    }
    /**
     * Update a user's details.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Alternative usage: User.updateCurrentUser
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp20
     * @param options Request options.
     */
    async updateCurrentUser(options) {
        return this.User.updateCurrentUser(options);
    }
    /**
     * Create a new project.
     * To include a custom avatar for the project, the project definition should contain an additional attribute with the key avatar and the value a data URI containing Base64-encoded image data. The URI should be in the following format:
     * ```
     * data:(content type, e.g. image/png);base64,(data)
     * ```
     * If the data is not Base64-encoded, or if a character set is defined in the URI, or the URI is otherwise invalid, project creation will fail.
     * The authenticated user must have PROJECT_CREATE permission to call this resource.
     * Alternative usage: Project.createProject
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp148
     * @param options Request options.
     */
    async createProject(options) {
        return this.Project.createProject(options);
    }
    /**
     * Retrieve a page of projects.
     * Only projects for which the authenticated user has the PROJECT_VIEW permission will be returned.
     * Alternative usage: Project.getProjects
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp149
     * @param options Request options.
     */
    async getProjects(options) {
        return this.Project.getProjects(options);
    }
    /**
     * Delete the project matching the supplied projectKey.
     * The authenticated user must have PROJECT_ADMIN permission for the specified project to call this resource.
     * Alternative usage: Project.deleteProject
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp151
     * @param options Request options.
     */
    async deleteProject(options) {
        return this.Project.deleteProject(options);
    }
    /**
     * Update the project matching the projectKey supplied in the resource path.
     * To include a custom avatar for the updated project, the project definition should contain an additional attribute with the key avatar and the value a data URI containing Base64-encoded image data. The URI should be in the following format: data:(content type, e.g. image/png);base64,(data) If the data is not Base64-encoded, or if a character set is defined in the URI, or the URI is otherwise invalid, project creation will fail.
     * The authenticated user must have PROJECT_ADMIN permission for the specified project to call this resource.
     * Alternative usage: Project.updateProject
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp152
     * @param options Request options.
     */
    async updateProject(options) {
        return this.Project.updateProject(options);
    }
    /**
     * Retrieve the project matching the supplied projectKey.
     * The authenticated user must have PROJECT_VIEW permission for the specified project to call this resource.
     * Alternative usage: Project.getProject
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp153
     * @param options Request options.
     */
    async getProject(options) {
        return this.Project.getProject(options);
    }
    /**
     * Create a new repository. Requires an existing project in which this repository will be created. The only parameters which will be used are name and scmId.
     * The authenticated user must have PROJECT_ADMIN permission for the context project to call this resource.
     * Alternative usage: Repository.createRepository
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp174
     * @param options Request options.
     */
    async createRepository(options) {
        return this.Repository.createRepository(options);
    }
    /**
     * Retrieve repositories from the project corresponding to the supplied projectKey.
     * The authenticated user must have REPO_READ permission for the specified project to call this resource.
     * Alternative usage: Repository.getRepositories
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp175
     * @param options Request options.
     */
    async getRepositories(options) {
        return this.Repository.getRepositories(options);
    }
    /**
     * Schedule the repository matching the supplied projectKey and repositorySlug to be deleted. If the request repository is not present
     * The authenticated user must have sufficient permissions specified by the repository delete policy to call this resource. The default permission required is REPO_ADMIN permission.
     * Alternative usage: Repository.deleteRepository
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp177
     * @param options Request options.
     */
    async deleteRepository(options) {
        return this.Repository.deleteRepository(options);
    }
    /**
     * Retrieve the repository matching the supplied projectKey and repositorySlug.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Alternative usage: Repository.getRepository
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp179
     * @param options Request options.
     */
    async getRepository(options) {
        return this.Repository.getRepository(options);
    }
    /**
     * Retrieve a page of pull requests to or from the specified repository.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource. Optionally clients can specify PR participant filters. Each filter has a mandatory username.N parameter, and the optional role.N and approved.N parameters.
     ** username.N - the "root" of a single participant filter, where "N" is a natural number starting from 1. This allows clients to specify multiple participant filters, by providing consecutive filters as username.1, username.2 etc. Note that the filters numbering has to start with 1 and be continuous for all filters to be processed. The total allowed number of participant filters is 10 and all filters exceeding that limit will be dropped.
     ** role.N(optional) the role associated with username.N. This must be one of AUTHOR, REVIEWER, orPARTICIPANT
     ** approved.N(optional) the approved status associated with username.N. That is whether username.N has approved the PR. Either true, or false
     * Alternative usage: Repository.PullRequest.getPullRequests
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp292
     * @param options Request options.
     */
    async getRepositoryPullRequests(options) {
        return this.Repository.PullRequest.getPullRequests(options);
    }
    /**
     * Create a new pull request between two branches. The branches may be in the same repository, or different ones. When using different repositories, they must still be in the same {@link Repository#getHierarchyId() hierarchy}.
     * The authenticated user must have REPO_READ permission for the "from" and "to"repositories to call this resource.
     * Alternative usage: Repository.PullRequest.createPullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp293
     * @param options Request options.
     */
    async createRepositoryPullRequest(options) {
        return this.Repository.PullRequest.createPullRequest(options);
    }
    /**
     * Retrieve a pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.getPullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp295
     * @param options Request options.
     */
    async getRepositoryPullRequest(options) {
        return this.Repository.PullRequest.getPullRequest(options);
    }
    /**
     * Update the title, description, reviewers or destination branch of an existing pull request.
     * Note: the reviewers list may be updated using this resource. However the author and participants list may not.
     * The authenticated user must either:
     ** be the author of the pull request and have the REPO_READ permission for the repository that this pull request targets; or
     ** have the REPO_WRITE permission for the repository that this pull request targets
     *
     * to call this resource.
     * Alternative usage: Repository.PullRequest.updatePullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp296
     * @param options Request options.
     */
    async updateRepositoryPullRequest(options) {
        return this.Repository.PullRequest.updatePullRequest(options);
    }
    /**
     * Deletes a pull request.
     * To call this resource, users must be authenticated and have permission to view the pull request. Additionally, they must:
     ** be the pull request author, if the system is configured to allow authors to delete their own pull requests (this is the default) OR
     ** have repository administrator permission for the repository the pull request is targeting
     *
     * A body containing the version of the pull request must be provided with this request.
     * Alternative usage: Repository.PullRequest.deletePullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp297
     * @param options Request options.
     */
    async deleteRepositoryPullRequest(options) {
        return this.Repository.PullRequest.deletePullRequest(options);
    }
    /**
     * Retrieve a page of activity associated with a pull request.
     * Activity items include comments, approvals, rescopes (i.e. adding and removing of commits), merges and more.
     * Different types of activity items may be introduced in newer versions of Stash or by user installed plugins, so clients should be flexible enough to handle unexpected entity shapes in the returned page.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.getActivities
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp303
     * @param options Request options.
     */
    async getRepositoryPullRequestActivities(options) {
        return this.Repository.PullRequest.getActivities(options);
    }
    /**
     * Decline a pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.declinePullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp305
     * @param options Request options.
     */
    async declineRepositoryPullRequest(options) {
        return this.Repository.PullRequest.declinePullRequest(options);
    }
    /**
     * Merge the specified pull request.
     * The authenticated user must have REPO_WRITE permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.mergePullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp308
     * @param options Request options.
     */
    async mergeRepositoryPullRequest(options) {
        return this.Repository.PullRequest.mergePullRequest(options);
    }
    /**
     * Re-open a declined pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.reopenPullRequest
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp310
     * @param options Request options.
     */
    async reopenRepositoryPullRequest(options) {
        return this.Repository.PullRequest.reopenPullRequest(options);
    }
    /**
     * Change the current user's status for a pull request. Implicitly adds the user as a participant if they are not already. If the current user is the author, this method will fail.
     * The possible values for status are UNAPPROVED, NEEDS_WORK, or APPROVED.
     * If the new status is NEEDS_WORK or APPROVED then the lastReviewedCommit for the participant will be updated to the latest commit of the source branch of the pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.changeStatus
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp345
     * @param options Request options.
     */
    async changeRepositoryPullRequestStatus(options) {
        return this.Repository.PullRequest.changeStatus(options);
    }
    /**
     * Gets comments for the specified pull request and path.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.Comment.getComments
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp324
     * @param options Request options.
     */
    async getRepositoryPullRequestComments(options) {
        return this.Repository.PullRequest.Comment.getComments(options);
    }
    /**
     * Add a new comment. Comments can be added in a few places by setting different attributes.
     * If the current user is not a participant the user is added as a watcher of the pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.Comment.createComment
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp325
     * @param options Request options.
     */
    async createRepositoryPullRequestComment(options) {
        return this.Repository.PullRequest.Comment.createComment(options);
    }
    /**
     * Delete a pull request comment. Anyone can delete their own comment. Only users with REPO_ADMIN and above may delete comments created by other users.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.Comment.deleteComment
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp327
     * @param options Request options.
     */
    async deleteRepositoryPullRequestComment(options) {
        return this.Repository.PullRequest.Comment.deleteComment(options);
    }
    /**
     * Update a comment, with the following restrictions:
     ** only the author of the comment may update the text of the comment
     ** only the author of the comment, the author of the pull request or repository admins and above may update the other fields of a comment
     *
     * Convert a comment to a task or vice versa. The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Note: the supplied JSON object must contain a version that must match the server's version of the comment or the update will fail. To determine the current version of the comment, the comment should be fetched from the server prior to the update. Look for the 'version' attribute in the returned JSON structure.
     * Alternative usage: Repository.PullRequest.Comment.updateComment
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp328
     * @param options Request options.
     */
    async updateRepositoryPullRequestComment(options) {
        return this.Repository.PullRequest.Comment.updateComment(options);
    }
    /**
     * Retrieves a pull request comment.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.Comment.getComment
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp329
     * @param options Request options.
     */
    async getRepositoryPullRequestComment(options) {
        return this.Repository.PullRequest.Comment.getComment(options);
    }
    /**
     * Retrieve commits for the specified pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Alternative usage: Repository.PullRequest.Commit.getCommits
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp333
     * @param options Request options.
     */
    async getRepositoryPullRequestCommits(options) {
        return this.Repository.PullRequest.Commit.getCommits(options);
    }
}
class RepositoryGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.PullRequest = new RepositoryPullRequestGroup(implementation, this.getGlobalErrorStrategy);
        this.Commit = new RepositoryCommitGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Create a new repository. Requires an existing project in which this repository will be created. The only parameters which will be used are name and scmId.
     * The authenticated user must have PROJECT_ADMIN permission for the context project to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp174
     * @param options Request options.
     */
    async createRepository(options) {
        const { projectKey, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieve repositories from the project corresponding to the supplied projectKey.
     * The authenticated user must have REPO_READ permission for the specified project to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp175
     * @param options Request options.
     */
    async getRepositories(options) {
        const { projectKey, limit, start, headers, errorStrategy } = options;
        const queryParams = { limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Schedule the repository matching the supplied projectKey and repositorySlug to be deleted. If the request repository is not present
     * The authenticated user must have sufficient permissions specified by the repository delete policy to call this resource. The default permission required is REPO_ADMIN permission.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp177
     * @param options Request options.
     */
    async deleteRepository(options) {
        const { projectKey, repositorySlug, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        try {
            return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
        }
        catch (error) {
            return {
                context: error.name,
                message: 'No repository matching the supplied projectKey and repositorySlug was found',
            };
        }
    }
    /**
     * Retrieve the repository matching the supplied projectKey and repositorySlug.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp179
     * @param options Request options.
     */
    async getRepository(options) {
        const { projectKey, repositorySlug, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RepositoryPullRequestGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Comment = new RepositoryPullRequestCommentGroup(implementation, this.getGlobalErrorStrategy);
        this.Commit = new RepositoryPullRequestCommitGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Retrieve a page of pull requests to or from the specified repository.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource. Optionally clients can specify PR participant filters. Each filter has a mandatory username.N parameter, and the optional role.N and approved.N parameters.
     ** username.N - the "root" of a single participant filter, where "N" is a natural number starting from 1. This allows clients to specify multiple participant filters, by providing consecutive filters as username.1, username.2 etc. Note that the filters numbering has to start with 1 and be continuous for all filters to be processed. The total allowed number of participant filters is 10 and all filters exceeding that limit will be dropped.
     ** role.N(optional) the role associated with username.N. This must be one of AUTHOR, REVIEWER, orPARTICIPANT
     ** approved.N(optional) the approved status associated with username.N. That is whether username.N has approved the PR. Either true, or false
     *
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp292
     * @param options Request options.
     */
    async getPullRequests(options) {
        const { projectKey, repositorySlug, at, direction, state, order, withAttributes, withProperties, filterText, limit, start, headers, errorStrategy } = options;
        const queryParams = { at, direction, state, order, withAttributes, withProperties, filterText, limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Create a new pull request between two branches. The branches may be in the same repository, or different ones. When using different repositories, they must still be in the same {@link Repository#getHierarchyId() hierarchy}.
     * The authenticated user must have REPO_READ permission for the "from" and "to"repositories to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp293
     * @param options Request options.
     */
    async createPullRequest(options) {
        const { projectKey, repositorySlug, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieve a pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp295
     * @param options Request options.
     */
    async getPullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Update the title, description, reviewers or destination branch of an existing pull request.
     * Note: the reviewers list may be updated using this resource. However the author and participants list may not.
     * The authenticated user must either:
     ** be the author of the pull request and have the REPO_READ permission for the repository that this pull request targets; or
     ** have the REPO_WRITE permission for the repository that this pull request targets
     *
     * to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp296
     * @param options Request options.
     */
    async updatePullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes a pull request.
     * To call this resource, users must be authenticated and have permission to view the pull request. Additionally, they must:
     ** be the pull request author, if the system is configured to allow authors to delete their own pull requests (this is the default) OR
     ** have repository administrator permission for the repository the pull request is targeting
     *
     * A body containing the version of the pull request must be provided with this request.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp297
     * @param options Request options.
     */
    async deletePullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, body, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Retrieve a page of activity associated with a pull request.
     * Activity items include comments, approvals, rescopes (i.e. adding and removing of commits), merges and more.
     * Different types of activity items may be introduced in newer versions of Stash or by user installed plugins, so clients should be flexible enough to handle unexpected entity shapes in the returned page.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp303
     * @param options Request options.
     */
    async getActivities(options) {
        const { projectKey, repositorySlug, pullRequestId, fromId, fromType, limit, start, headers, errorStrategy } = options;
        const queryParams = { fromId, fromType, limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/activities`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Decline a pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp305
     * @param options Request options.
     */
    async declinePullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, version, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/decline?version=${encodeURIComponent(version)}`, constructHeaders(null, { 'Content-Type': 'application/json', ...headers }), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Merge the specified pull request.
     * The authenticated user must have REPO_WRITE permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp308
     * @param options Request options.
     */
    async mergePullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, version, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/merge?version=${encodeURIComponent(version)}`, constructHeaders(null, { 'Content-Type': 'application/json', ...headers }), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Re-open a declined pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp310
     * @param options Request options.
     */
    async reopenPullRequest(options) {
        const { projectKey, repositorySlug, pullRequestId, version, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/reopen?version=${encodeURIComponent(version)}`, constructHeaders(null, { 'Content-Type': 'application/json', ...headers }), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Change the current user's status for a pull request. Implicitly adds the user as a participant if they are not already. If the current user is the author, this method will fail.
     * The possible values for status are UNAPPROVED, NEEDS_WORK, or APPROVED.
     * If the new status is NEEDS_WORK or APPROVED then the lastReviewedCommit for the participant will be updated to the latest commit of the source branch of the pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp345
     * @param options Request options.
     */
    async changeStatus(options) {
        const { projectKey, repositorySlug, pullRequestId, userSlug, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/participants/${encodeURIComponent(userSlug)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RepositoryCommitGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * This is a paged API. This API can also be invoked via a user-centric URL when addressing repositories in personal projects.
     * Retrieve a page of commits from a given starting commit or "between" two commits.
     * If no explicit commit is specified, the tip of the repository's default branch is assumed.
     * Commits may be identified by branch or tag name or by ID.
     * A path may be supplied to restrict the returned commits to only those which affect that path.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp222
     * @param options Request options.
     */
    async getRepositoryCommits(options) {
        const { projectKey, repositorySlug, followRenames, ignoreMissing, merges, path, since, until, withCounts, avatarSize, avatarScheme, limit, start, headers, errorStrategy } = options;
        const queryParams = { followRenames, ignoreMissing, merges, path, since, until, withCounts, avatarSize, avatarScheme, limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/commits`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * This API can also be invoked via a user-centric URL when addressing repositories in personal projects.
     * Retrieve a single commit identified by its ID. In general, that ID is a SHA1. From 2.11, ref names like "refs/heads/master" are no longer accepted by this resource.
     * The authenticated user must have REPO_READ permission for the specified repository to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp224
     * @param options Request options.
     */
    async getCommit(options) {
        const { projectKey, repositorySlug, commitId, path, headers, errorStrategy } = options;
        const queryParams = { path };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/commits/${encodeURIComponent(commitId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RepositoryPullRequestCommentGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets comments for the specified pull request and path.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp324
     * @param options Request options.
     */
    async getComments(options) {
        const { projectKey, repositorySlug, pullRequestId, anchorState, diffType, fromHash, toHash, state, path, limit, start, headers, errorStrategy } = options;
        const queryParams = { anchorState, diffType, fromHash, toHash, state, path, limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/comments`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Add a new comment. Comments can be added in a few places by setting different attributes.
     * If the current user is not a participant the user is added as a watcher of the pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp325
     * @param options Request options.
     */
    async createComment(options) {
        const { projectKey, repositorySlug, pullRequestId, body, headers, errorStrategy } = options;
        const request = new Request('POST', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/comments`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete a pull request comment. Anyone can delete their own comment. Only users with REPO_ADMIN and above may delete comments created by other users.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp327
     * @param options Request options.
     */
    async deleteComment(options) {
        const { projectKey, repositorySlug, pullRequestId, commentId, version, headers, errorStrategy } = options;
        const queryParams = { version };
        const request = new Request('DELETE', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/comments/${encodeURIComponent(commentId)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Update a comment, with the following restrictions:
     ** only the author of the comment may update the text of the comment
     ** only the author of the comment, the author of the pull request or repository admins and above may update the other fields of a comment
     *
     * Convert a comment to a task or vice versa. The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Note: the supplied JSON object must contain a version that must match the server's version of the comment or the update will fail. To determine the current version of the comment, the comment should be fetched from the server prior to the update. Look for the 'version' attribute in the returned JSON structure.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp328
     * @param options Request options.
     */
    async updateComment(options) {
        const { projectKey, repositorySlug, pullRequestId, commentId, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/comments/${encodeURIComponent(commentId)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieves a pull request comment.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp329
     * @param options Request options.
     */
    async getComment(options) {
        const { projectKey, repositorySlug, pullRequestId, commentId, headers, errorStrategy } = options;
        const request = new Request('GET', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/comments/${encodeURIComponent(commentId)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class RepositoryPullRequestCommitGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Retrieve commits for the specified pull request.
     * The authenticated user must have REPO_READ permission for the repository that this pull request targets to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp333
     * @param options Request options.
     */
    async getCommits(options) {
        const { projectKey, repositorySlug, pullRequestId, avatarScheme, avatarSize, withCounts, limit, start, headers, errorStrategy } = options;
        const queryParams = { avatarScheme, avatarSize, withCounts, limit, start };
        const request = new Request('GET', constructUrl(`/rest/api/1.0/projects/${encodeURIComponent(projectKey)}/repos/${encodeURIComponent(repositorySlug)}/pull-requests/${encodeURIComponent(pullRequestId)}/commits`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class ProjectGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Create a new project.
     * To include a custom avatar for the project, the project definition should contain an additional attribute with the key avatar and the value a data URI containing Base64-encoded image data. The URI should be in the following format:
     * ```
     * data:(content type, e.g. image/png);base64,(data)
     * ```
     * If the data is not Base64-encoded, or if a character set is defined in the URI, or the URI is otherwise invalid, project creation will fail.
     * The authenticated user must have PROJECT_CREATE permission to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp148
     * @param options Request options.
     */
    async createProject(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('POST', '/rest/api/1.0/projects', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieve a page of projects.
     * Only projects for which the authenticated user has the PROJECT_VIEW permission will be returned.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp149
     * @param options Request options.
     */
    async getProjects(options) {
        const { name, permission, limit, start, headers, errorStrategy } = options ?? {};
        const queryParams = { name, permission, limit, start };
        const request = new Request('GET', constructUrl('/rest/api/1.0/projects', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Delete the project matching the supplied projectKey.
     * The authenticated user must have PROJECT_ADMIN permission for the specified project to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp151
     * @param options Request options.
     */
    async deleteProject(options) {
        const { projectKey, headers, errorStrategy } = options;
        const request = new Request('DELETE', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Update the project matching the projectKey supplied in the resource path.
     * To include a custom avatar for the updated project, the project definition should contain an additional attribute with the key avatar and the value a data URI containing Base64-encoded image data. The URI should be in the following format: data:(content type, e.g. image/png);base64,(data) If the data is not Base64-encoded, or if a character set is defined in the URI, or the URI is otherwise invalid, project creation will fail.
     * The authenticated user must have PROJECT_ADMIN permission for the specified project to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp152
     * @param options Request options.
     */
    async updateProject(options) {
        const { projectKey, body, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieve the project matching the supplied projectKey.
     * The authenticated user must have PROJECT_VIEW permission for the specified project to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp153
     * @param options Request options.
     */
    async getProject(options) {
        const { projectKey, headers, errorStrategy } = options;
        const request = new Request('PUT', `/rest/api/1.0/projects/${encodeURIComponent(projectKey)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class UserGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Retrieve a page of users.
     * The authenticated user must have the LICENSED_USER permission to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp17
     * @param options Request options.
     */
    async getUsers(options) {
        const { filter, headers, errorStrategy } = options ?? {};
        const queryParams = { filter };
        const request = new Request('GET', constructUrl('/rest/api/1.0/admin/users', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Deletes the specified user, removing them from the system. This also removes any permissions that may have been granted to the user.
     * A user may not delete themselves, and a user with ADMIN permissions may not delete a user with SYS_ADMINpermissions.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp18
     * @param options Request options.
     */
    async deleteUser(options) {
        const { name, headers, errorStrategy } = options;
        const queryParams = { name };
        const request = new Request('DELETE', constructUrl('/rest/api/1.0/admin/users', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Creates a new user from the assembled query parameters.
     * The default group can be used to control initial permissions for new users, such as granting users the ability to login or providing read access to certain projects or repositories. If the user is not added to the default group, they may not be able to login after their account is created until explicit permissions are configured.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp19
     * @param options Request options.
     */
    async createUser(options) {
        const { displayName, emailAddress, name, password, addToDefaultGroup, notify, headers, errorStrategy } = options;
        const queryParams = { displayName, emailAddress, name, password, addToDefaultGroup, notify };
        const request = new Request('POST', constructUrl('/rest/api/1.0/admin/users', queryParams), constructHeaders(null, { 'Content-Type': 'application/json', ...headers }), null);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Update a user's details.
     * The authenticated user must have the ADMIN permission to call this resource.
     * Documentation: https://docs.atlassian.com/bitbucket-server/rest/7.11.1/bitbucket-rest.html#idp20
     * @param options Request options.
     */
    async updateCurrentUser(options) {
        const { body, headers, errorStrategy } = options;
        const request = new Request('PUT', '/rest/api/1.0/admin/users', constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
